package Data::Query;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Data::Query - Mixin class for objects querying various data resources

=head1 SYNOPSIS

 use Data::Query;

=head1 DESCRIPTION

This class serves as a base class for objects capable of querying various data
storages (like databases or program structures) using the Data::Filter::Rules. This
class defines the mandatory interface, which must be implemented, and provides useful
features like comparator or conversion functions.

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #--- DEVELOPMENT ONLY ---#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item query($;@) [PUBLIC]

  Usage     : my $mentat_messages = $instance->query($filter)
  Purpose   : Query the destination for Mentat messages which pass the given filter
  Arguments : Data::Filter $filter - query filter tree
              Additional arguments according to the specific implementation
  Returns   : ARRAY REFERENCE containing result
  Throws    : Croaks, if not invoked on object

=cut

sub query($;@)
{
    my $self = shift;
    confess "Method needs implementation";
    my ($filter, @args) = @_;
}

#-------------------------------------------------------------------------------

=item convertor(;$) [PUBLIC]

  Usage     :
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Croaks, if not invoked on object

=cut

sub convertor(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($convertor,) = @_;

    # Setter mode
    if ($convertor) {
        confess "Convertor object must be given as argument" unless $convertor and blessed($convertor);
        $self->{CONVERTOR} = $convertor;
    }
    return $self->{CONVERTOR};
}

=item convert($$) [PUBLIC]

  Usage     : my $result = $instance->convert($conversion, $value)
  Purpose   : Perform the designated conversion using the inner convertor
  Arguments : STRING $conversion - name of the conversion method
              MIXED @args - arguments for conversion including the original value
  Returns   : MIXED $result
  Throws    : Croaks, if not invoked on object
  Comments  : Indirectly executes the inner convertor methods
  See Also  : Mentat::Message::Convertor based convertors

=cut

sub convert($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($conversion, @args) = @_;

    return $self->convertor->$conversion(@args);
}


=pod

=back

=cut

1;
