package Mentat::Message::Value::Mongo;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Message::Value::Mongo - Utility class for determining datatypes of Mentat message values

=head1 SYNOPSIS

 use Mentat::Message::Value::Mongo;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Message::Value;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# Table containing datatypes of IDEA elements on given paths
my %MPATH2DATATYPE = (
        'Alert/@messageid'              => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@analyzerid'    => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@name'          => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@manufacturer'  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@model'         => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@version'       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@class'         => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@ostype'        => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/@osversion'     => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Analyzer/Node/@ident'                => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Node/@category'             => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Analyzer/Node/location'              => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Node/name'                  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Node/Address/@ident'        => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Node/Address/@category'     => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Analyzer/Node/Address/@vlan_name'    => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Node/Address/@vlan_num'     => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Analyzer/Node/Address/address'       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Node/Address/netmask'       => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Analyzer/Process/@ident' => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Process/name'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Process/pid'    => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Analyzer/Process/path'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Process/arg'    => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Analyzer/Process/env'    => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/CreateTime/@ntpstamp'    => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Alert/CreateTime'              => Mentat::Message::Value::DATATYPE_DATETIME,

        'Alert/DetectTime/@ntpstamp'    => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Alert/DetectTime'              => Mentat::Message::Value::DATATYPE_DATETIME,

        'Alert/AnalyzerTime/@ntpstamp'  => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Alert/AnalyzerTime'            => Mentat::Message::Value::DATATYPE_DATETIME,

        'Alert/Classification/@ident'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Classification/@text'    => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Classification/Reference/@origin'    => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Classification/Reference/@meaning'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Classification/Reference/name'       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Classification/Reference/url'        => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Source/@ident'       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/@spoofed'     => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Source/@interface'   => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Source/Node/@ident'              => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Node/@category'           => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Source/Node/location'            => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Node/name'                => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Node/Address/@ident'      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Node/Address/@category'   => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Source/Node/Address/@vlan_name'  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Node/Address/@vlan_num'   => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Node/Address/address'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Node/Address/netmask'     => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Source/User/@ident'          => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/User/@category'       => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Source/User/UserId/@ident'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/User/UserId/@type'    => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Source/User/UserId/@tty'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/User/UserId/name'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/User/UserId/number'   => Mentat::Message::Value::DATATYPE_INT,

        'Alert/Source/Process/@ident'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Process/name'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Process/pid'      => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Process/path'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Process/arg'      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Process/env'      => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Source/Service/@ident'                               => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/@ip_version'                          => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Service/@iana_protocol_number'                => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Service/@iana_protocol_name'                  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/name'                                 => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/port'                                 => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Service/portlist'                             => Mentat::Message::Value::DATATYPE_PORTLIST,
        'Alert/Source/Service/protocol'                             => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/WebService/url'                       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/WebService/cgi'                       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/WebService/http_method'               => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/WebService/arg'                       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/SNMPService/oid'                      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/SNMPService/message_processing_model' => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Service/SNMPService/security_model'           => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Service/SNMPService/security_name'            => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/SNMPService/security_level'           => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Source/Service/SNMPService/context_name'             => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/SNMPService/context_engine_id'        => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Source/Service/SNMPService/command'                  => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Target/@ident'       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/@decoy'       => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/@interface'   => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Target/Node/@ident'              => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Node/@category'           => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/Node/location'            => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Node/name'                => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Node/Address/@ident'      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Node/Address/@category'   => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/Node/Address/@vlan_name'  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Node/Address/@vlan_num'   => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Node/Address/address'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Node/Address/netmask'     => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Target/User/@ident'          => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/User/@category'       => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/User/UserId/@ident'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/User/UserId/@type'    => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/User/UserId/@tty'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/User/UserId/name'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/User/UserId/number'   => Mentat::Message::Value::DATATYPE_INT,

        'Alert/Target/Process/@ident'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Process/name'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Process/pid'      => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Process/path'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Process/arg'      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Process/env'      => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Target/Service/@ident'                               => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/@ip_version'                          => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Service/@iana_protocol_number'                => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Service/@iana_protocol_name'                  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/name'                                 => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/port'                                 => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Service/portlist'                             => Mentat::Message::Value::DATATYPE_PORTLIST,
        'Alert/Target/Service/protocol'                             => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/WebService/url'                       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/WebService/cgi'                       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/WebService/http_method'               => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/WebService/arg'                       => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/SNMPService/oid'                      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/SNMPService/message_processing_model' => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Service/SNMPService/security_model'           => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Service/SNMPService/security_name'            => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/SNMPService/security_level'           => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/Service/SNMPService/context_name'             => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/SNMPService/context_engine_id'        => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/Service/SNMPService/command'                  => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Target/File/@ident'                          => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/@category'                       => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/File/@fstype'                         => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/File/@file-type'                      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/name'                            => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/path'                            => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/create-time'                     => Mentat::Message::Value::DATATYPE_DATETIME,
        'Alert/Target/File/modify-time'                     => Mentat::Message::Value::DATATYPE_DATETIME,
        'Alert/Target/File/access-time'                     => Mentat::Message::Value::DATATYPE_DATETIME,
        'Alert/Target/File/data-size'                       => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/disk-size'                       => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/FileAccess/UserId/@ident'        => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/FileAccess/UserId/@type'         => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/File/FileAccess/UserId/@tty'          => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/FileAccess/UserId/name'          => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/FileAccess/UserId/number'        => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/FileAccess/Permission/@perms'    => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/File/Linkage/@category'               => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/File/Linkage/name'                    => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/Linkage/path'                    => Mentat::Message::Value::DATATYPE_STRING,
        #'Alert/Target/File/Linkage/File'                   => Mentat::Message::Value::DATATYPE_,
        'Alert/Target/File/Inode/change-time'               => Mentat::Message::Value::DATATYPE_DATETIME,
        'Alert/Target/File/Inode/number'                    => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/Inode/major-device'              => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/Inode/minor-device'              => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/Inode/c-major-device'            => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/Inode/c-minor-device'            => Mentat::Message::Value::DATATYPE_INT,
        'Alert/Target/File/Checksum/@algorithm'             => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Target/File/Checksum/value'                  => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/Target/File/Checksum/key'                    => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Assessment/Impact/@severity'     => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Assessment/Impact/@completion'   => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Assessment/Impact/@type'         => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Assessment/Impact'               => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Assessment/Action/@category' => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Assessment/Action'           => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/Assessment/Confidence/rating'    => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/Assessment/Confidence'           => Mentat::Message::Value::DATATYPE_REAL,

        'Alert/AdditionalData/@type'        => Mentat::Message::Value::DATATYPE_ENUM,
        'Alert/AdditionalData/@meaning'     => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/AdditionalData/boolean'      => Mentat::Message::Value::DATATYPE_BOOL,
        'Alert/AdditionalData/byte'         => Mentat::Message::Value::DATATYPE_BYTE,
        'Alert/AdditionalData/character'    => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/AdditionalData/date-time'    => Mentat::Message::Value::DATATYPE_DATETIME,
        'Alert/AdditionalData/integer'      => Mentat::Message::Value::DATATYPE_INT,
        'Alert/AdditionalData/ntpstamp'     => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Alert/AdditionalData/portlist'     => Mentat::Message::Value::DATATYPE_PORTLIST,
        'Alert/AdditionalData/real'         => Mentat::Message::Value::DATATYPE_REAL,
        'Alert/AdditionalData/string'       => Mentat::Message::Value::DATATYPE_STRING,
        #'Alert/AdditionalData/byte-string'     => Mentat::Message::Value::DATATYPE_,
        #'Alert/AdditionalData/xmltext'         => Mentat::Message::Value::DATATYPE_,

        'Alert/ToolAlert/name'                      => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/ToolAlert/command'                   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/ToolAlert/alertident/@analyzerid'    => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/ToolAlert/alertident/alertident'     => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/CorrelationAlert/name'                   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/CorrelationAlert/alertident/@analyzerid' => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/CorrelationAlert/alertident/alertident'  => Mentat::Message::Value::DATATYPE_STRING,

        'Alert/OverflowAlert/program'   => Mentat::Message::Value::DATATYPE_STRING,
        'Alert/OverflowAlert/size'      => Mentat::Message::Value::DATATYPE_INT,
        'Alert/OverflowAlert/buffer'    => Mentat::Message::Value::DATATYPE_BYTE,

        'Heartbeat/@messageid'              => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@analyzerid'    => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@name'          => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@manufacturer'  => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@model'         => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@version'       => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@class'         => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@ostype'        => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/@osversion'     => Mentat::Message::Value::DATATYPE_STRING,

        'Heartbeat/Analyzer/Node/@ident'                => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Node/@category'             => Mentat::Message::Value::DATATYPE_ENUM,
        'Heartbeat/Analyzer/Node/location'              => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Node/name'                  => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Node/Address/@ident'        => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Node/Address/@category'     => Mentat::Message::Value::DATATYPE_ENUM,
        'Heartbeat/Analyzer/Node/Address/@vlan_name'    => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Node/Address/@vlan_num'     => Mentat::Message::Value::DATATYPE_INT,
        'Heartbeat/Analyzer/Node/Address/address'       => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Node/Address/netmask'       => Mentat::Message::Value::DATATYPE_STRING,

        'Heartbeat/Analyzer/Process/@ident' => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Process/name'   => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Process/pid'    => Mentat::Message::Value::DATATYPE_INT,
        'Heartbeat/Analyzer/Process/path'   => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Process/arg'    => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/Analyzer/Process/env'    => Mentat::Message::Value::DATATYPE_STRING,

        'Heartbeat/CreateTime/@ntpstamp'    => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Heartbeat/CreateTime'              => Mentat::Message::Value::DATATYPE_DATETIME,

        'Heartbeat/HeartbeatInterval'   => Mentat::Message::Value::DATATYPE_INT,

        'Heartbeat/AnalyzerTime/@ntpstamp'  => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Heartbeat/AnalyzerTime'            => Mentat::Message::Value::DATATYPE_DATETIME,

        'Heartbeat/AdditionalData/@type'        => Mentat::Message::Value::DATATYPE_ENUM,
        'Heartbeat/AdditionalData/@meaning'     => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/AdditionalData/boolean'      => Mentat::Message::Value::DATATYPE_BOOL,
        'Heartbeat/AdditionalData/byte'         => Mentat::Message::Value::DATATYPE_BYTE,
        'Heartbeat/AdditionalData/character'    => Mentat::Message::Value::DATATYPE_STRING,
        'Heartbeat/AdditionalData/date-time'    => Mentat::Message::Value::DATATYPE_DATETIME,
        'Heartbeat/AdditionalData/integer'      => Mentat::Message::Value::DATATYPE_INT,
        'Heartbeat/AdditionalData/ntpstamp'     => Mentat::Message::Value::DATATYPE_NTPSTAMP,
        'Heartbeat/AdditionalData/portlist'     => Mentat::Message::Value::DATATYPE_PORTLIST,
        'Heartbeat/AdditionalData/real'         => Mentat::Message::Value::DATATYPE_REAL,
        'Heartbeat/AdditionalData/string'       => Mentat::Message::Value::DATATYPE_STRING,
        #'Heartbeat/AdditionalData/byte-string'     => Mentat::Message::Value::DATATYPE_,
        #'Heartbeat/AdditionalData/xmltext'         => Mentat::Message::Value::DATATYPE_,
    );

# Datatype of some elements is dependent on value of some other objects
my %DEPENDENT_DATATYPE = (
        'Alert/Analyzer/Node/Address/address'   => 'Alert/Analyzer/Node/Address/@category',
        'Alert/Source/Node/Address/address'     => 'Alert/Source/Node/Address/@category',
        'Alert/Target/Node/Address/address'     => 'Alert/Target/Node/Address/@category',

        'Heartbeat/Analyzer/Node/Address/address'   => 'Heartbeat/Analyzer/Node/Address/@category',
    );

# Table containing datatypes of dependent elements
my %DEPENDENT2DATATYPE = (
        'e-mail'        => Mentat::Message::Value::DATATYPE_EMAIL,
        'mac'           => Mentat::Message::Value::DATATYPE_MAC,
        'ipv4-addr'     => Mentat::Message::Value::DATATYPE_IPV4_STR,
        'ipv4-addr-hex' => Mentat::Message::Value::DATATYPE_IPV4_HEX,
        'ipv4-net'      => Mentat::Message::Value::DATATYPE_IPV4_STR_NET,
        'ipv4-net-mask' => Mentat::Message::Value::DATATYPE_IPV4_STR_NET_MASK,
        'ipv6-addr'     => Mentat::Message::Value::DATATYPE_IPV6_STR,
        'ipv6-addr-hex' => Mentat::Message::Value::DATATYPE_IPV6_HEX,
        'ipv6-net'      => Mentat::Message::Value::DATATYPE_IPV6_STR_NET,
        'ipv6-net-mask' => Mentat::Message::Value::DATATYPE_IPV6_STR_NET_MASK,
    );

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION    = '0.1';
    $DEVEL      = 0;
    @ISA        = ('Mentat::Message::Value');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item path_datatype($) [PUBLIC,STATIC]

  Usage     : my $datatype = Mentat::Message::Value->path_datatype($mpath)
  Purpose   : Determine the data type of the given MPath node
  Arguments : STRING containing MPath
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub path_datatype($)
{
    my $class = shift;
    croak ((caller(0))[3] . ": class method invoked on object") if ref $class;
    my ($mpath,) = @_;
    croak ((caller(0))[3] . ": MPath must be given as argument") unless $mpath;
    croak ((caller(0))[3] . ": unknown MPath '$mpath'") unless exists $MPATH2DATATYPE{$mpath};

    return $MPATH2DATATYPE{$mpath};
}

=item value_datatype($$) [PUBLIC,STATIC]

  Usage     : my $datatype = Mentat::Message::Value->value_datatype($mpath)
  Purpose   : Determine the data type of the value on given MPath node in given IDEA message
  Arguments : STRING containing MPath
              Mentat::Message::IDEA reference
  Returns   :
  Throws    : Croaks, if invoked on object
  Comments  :
  See Also  :

=cut

sub value_datatype($$)
{
    my $class = shift;
    croak ((caller(0))[3] . ": class method invoked on object") if ref $class;
    my ($mpath, $mentat_message) = @_;
    croak ((caller(0))[3] . ": MPath must be given as argument") unless $mpath;
    croak ((caller(0))[3] . ": unknown MPath '$mpath'") unless exists $MPATH2DATATYPE{$mpath};

    if ($DEPENDENT_DATATYPE{$mpath})
    {
        # TODO
    }
    else {
        return $MPATH2DATATYPE{$mpath};
    }
}

=pod

=back

=cut

1;
