package Mentat::Processor::Engine::FileStore;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::FileStore - Engine for storing ALERT messages into files

=head1 SYNOPSIS

 use Mentat::Processor::Engine::FileStore;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Michal Kostenec
 Cesnet, z.s.p.o
 kostenec@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Mentat::Message::Factory;
use Mentat::Processor::Queue::Folder;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process($$) [PUBLIC]

  Usage     : Emit/execute event 'message_hello' with appropriate arguments
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;
    confess "Message must be a 'Mentat::Processor::Message' implementation" unless blessed($message) and $message->isa('Mentat::Processor::Message');

    # Store the message to the filesystem
    my $queue_id = $self->{STORE}->enqueue_msg($message->alert_obj);

    # Check for success
    if ($queue_id) {
        $self->statistics->{'cnt_stored'}++;
        Log::Loger->info(__PACKAGE__, "[ENG:FileStore] Message saved with ID '".$queue_id->[0]."'");
        $self->emit_event('message_commit', $message);
        return (EC_CONTINUE, $message);
    }

    # Deal with failure
    $self->statistics->{'cnt_errors'}++;
    Log::Loger->error(__PACKAGE__, "[ENG:FileStore] Storing of message '$message' failed");
    $self->emit_event('message_banish', $message);

    return EC_STOP;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Engine instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[ENG:FileStore] Constructor expects configurations as argument\n" unless %config;
    die "[ENG:FileStore] FileStore engine configuration 'store_folder' is missing\n" unless $config{store_folder};
    die "[ENG:FileStore] FileStore engine configuration 'temp_folder' is missing\n" unless $config{temp_folder};

    Log::Loger->info(__PACKAGE__, "[ENG:FileStore] Using folder '$config{store_folder}' as store folder");
    Log::Loger->info(__PACKAGE__, "[ENG:FileStore] Using folder '$config{temp_folder}' as temp folder");
    Log::Loger->info(__PACKAGE__, "[ENG:FileStore] Using folder '$config{error_folder}' as error folder");

    $self->{STORE} = Mentat::Processor::Queue::Folder->new(
                        incoming_folder => $config{store_folder},
                        pending_folder  => $config{store_folder},
                        temp_folder     => $config{temp_folder},
                        error_folder    => $config{error_folder});

    return $self;
}

=pod

=back

=cut

1;
